@@ -2,13 +2,15 @@
 
 #ifndef __CONFIG_H
 #define __CONFIG_H
+#define GCC
+#define HAVE_EXR 0
 #define HAVE_JPEG 1
 #define HAVE_PTHREAD 1
 #define HAVE_ZLIB 1
 #define LIBPATH "/usr/local/lib"
-#define MIN_RAYDIST 0.000005
+#define MIN_RAYDIST 0.00005
 #define PACKAGE "yafray"
-#define VERSION "0.0.6"
+#define VERSION "0.0.8"
 
 #define __BEGIN_YAFRAY namespace yafray {
 #define __END_YAFRAY }
@@ -18,4 +20,6 @@ typedef float CFLOAT;
 typedef float GFLOAT;
 typedef float PFLOAT;
 __END_YAFRAY
+#define YAFRAYPLUGIN_EXPORT
+#define YAFRAYCORE_EXPORT
 #endif
