@@ -22,9 +22,21 @@
 #include "interface_impl.h"
 
 #include "sphere.h"
+
+#if HAVE_PTHREAD
+#include "threadedscene.h"
+#endif
+
 #include "mesh.h"
 #include "reference.h"
 
+#include "targaIO.h"
+#include "HDR_io.h"
+
+#if HAVE_EXR
+#include "EXR_io.h"
+#endif
+
 #include "yafsystem.h"
 
 using namespace std;
@@ -178,10 +190,10 @@ void interfaceImpl_t::addShader(paramMap_t &params,list<paramMap_t> &lparams)
 
 void interfaceImpl_t::addObject_trimesh(const std::string &name,
 				std::vector<point3d_t> &verts, const std::vector<int> &faces,
-				std::vector<GFLOAT> &facesuv,std::vector<CFLOAT> &vcol,
-				const std::vector<std::string> &shaders,const std::vector<int> &faceshader,
-				float sm_angle,bool castShadows,bool useR,bool receiveR,bool caus,bool has_orco,
-				const color_t &caus_rcolor,const color_t &caus_tcolor,float caus_IOR)
+				std::vector<GFLOAT> &facesuv, std::vector<CFLOAT> &vcol,
+				const std::vector<std::string> &shaders, const std::vector<int> &faceshader,
+				float sm_angle, bool castShadows, bool useR, bool receiveR, bool caus, bool has_orco,
+				const color_t &caus_rcolor, const color_t &caus_tcolor, float caus_IOR)
 {
 	string shader;
 	if(shaders.size()>0) shader=shaders[0];
@@ -228,11 +240,12 @@ void interfaceImpl_t::addObject_trimesh(const std::string &name,
 			rfaces[i].setShader(shader_pointer[faceshader[i]]);
 
 	vector<vector3d_t> normals;
-	obj=meshObject_t::factory(verts,normals,rfaces,facesuv,vcol);
-	obj->hasOrco(has_orco);
-	obj->transform(M);
+	obj=meshObject_t::factory(has_orco, M, verts,normals,rfaces,facesuv,vcol);
+	//obj->hasOrco(has_orco);
+	//obj->transform(M);
 	if(sm_angle>0.0)
 		obj->autoSmooth(sm_angle);
+	obj->tangentsFromUV();
 
 	if(object_table.find(name)!=object_table.end())
 	{
@@ -243,13 +256,12 @@ void interfaceImpl_t::addObject_trimesh(const std::string &name,
 	obj->useForRadiosity(useR);
 	obj->reciveRadiosity(receiveR);
 	obj->caustics(caus);
-	obj->setCaustic(caus_rcolor,caus_tcolor,caus_IOR);
+	obj->setCaustic(caus_rcolor, caus_tcolor, caus_IOR);
 	obj->setShader(shader_table[shader]);
 	object_table[name]=obj;
 	INFO<<"Added object "<<name<<endl;
 }
 
-
 void interfaceImpl_t::addObject_reference(const string &name,const string &original)
 {
 	object3d_t *obj=NULL;
@@ -305,13 +317,16 @@ void interfaceImpl_t::addLight(paramMap_t &params)
 
 void interfaceImpl_t::addCamera(paramMap_t &params)
 {
-	string _name, _type="perspective";
-	const string *name=&_name, *type=&_type;
+	string _name, _type="perspective",
+			_bkhtype="disk1", _bkhbias="uniform";
+	const string *name=&_name, *type=&_type,
+			*bkhtype=&_bkhtype, *bkhbias=&_bkhbias;
 	point3d_t from(0,1,0), to(0,0,0), up(0,1,1);
 	int resx=320, resy=200;
-	float dfocal=1, apt=0, dofd=0;
+	float aspect=1, dfocal=1, apt=0, dofd=0, bkhrot=0;
 	bool useq = false;
 	params.getParam("name", name);
+	if (*name=="") return;
 	params.getParam("from", from);
 	params.getParam("to", to);
 	params.getParam("up", up);
@@ -322,10 +337,42 @@ void interfaceImpl_t::addCamera(paramMap_t &params)
 	params.getParam("dof_distance", dofd);
 	params.getParam("use_qmc", useq);
 	params.getParam("type", type);
-	if (*name=="") return;
-	cameraType ct = CAM_PERSPECTIVE;
-	if (*type=="ortho") ct = CAM_ORTHO;
-	camera_t *cam=new camera_t(from, to, up, resx, resy, dfocal, apt, dofd, useq, ct);
+	params.getParam("bokeh_type", bkhtype);
+	params.getParam("bokeh_bias", bkhbias);
+	params.getParam("bokeh_rotation", bkhrot);
+	params.getParam("aspect_ratio", aspect);
+	// camera type
+	camera_t::cameraType ct = camera_t::CM_PERSPECTIVE;
+	if (*type=="ortho")
+		ct = camera_t::CM_ORTHO;
+	else if (*type=="spherical")
+		ct = camera_t::CM_SPHERICAL;
+	else if (*type=="lightprobe")
+		ct = camera_t::CM_LIGHTPROBE;
+	// bokeh type
+	camera_t::bokehType bt = camera_t::BK_DISK1;
+	if (*bkhtype=="disk2")
+		bt = camera_t::BK_DISK2;
+	else if (*bkhtype=="triangle")
+		bt = camera_t::BK_TRI;
+	else if (*bkhtype=="square")
+		bt = camera_t::BK_SQR;
+	else if (*bkhtype=="pentagon")
+		bt = camera_t::BK_PENTA;
+	else if (*bkhtype=="hexagon")
+		bt = camera_t::BK_HEXA;
+	else if (*bkhtype=="ring")
+		bt = camera_t::BK_RING;
+	// bokeh bias
+	camera_t::bkhBiasType bbt = camera_t::BB_NONE;
+	if (*bkhbias=="center")
+		bbt = camera_t::BB_CENTER;
+	else if (*bkhbias=="edge")
+		bbt = camera_t::BB_EDGE;
+	camera_t *cam=new camera_t(from, to, up,
+														resx, resy, aspect, dfocal,
+														apt, dofd, useq,
+														ct, bt, bbt, bkhrot);
 	if (camera_table.find(*name)!=camera_table.end())
 	{
 		WARNING << "Camera " << name << " redefined\n";
@@ -434,6 +481,7 @@ void interfaceImpl_t::render(paramMap_t &params)
 	const string *camera=&_camera, *outfile=&_outfile,*background=&_background;
 	int raydepth = 3;
 	PFLOAT bias = 0.1;
+	GFLOAT xmin=-1.5,xmax=1.5,ymin=-1.5,ymax=1.5;
 	bool save_alpha = false;
 
 	params.getParam("camera_name", camera);
@@ -442,6 +490,10 @@ void interfaceImpl_t::render(paramMap_t &params)
 	params.getParam("raydepth", raydepth);
 	params.getParam("bias", bias);
 	params.getParam("background_name", background);
+	params.getParam("border_xmin",xmin);
+	params.getParam("border_xmax",xmax);
+	params.getParam("border_ymin",ymin);
+	params.getParam("border_ymax",ymax);
 
 	// get new AA parameters
 	int AA_passes=0, AA_minsamples=1;
@@ -460,8 +512,7 @@ void interfaceImpl_t::render(paramMap_t &params)
 		INFO << "AA_minsamples must be at least 1\n";
 		AA_minsamples = 1;
 	}
-	PFLOAT AA_pixelwidth=1.0, AA_threshold=0.125;
-	int AA_onepass_max = 0;
+	PFLOAT AA_pixelwidth=1.5, AA_threshold=0.05;
 	params.getParam("AA_pixelwidth", AA_pixelwidth);
 	if (AA_pixelwidth<1.0) {
 	  INFO << "AA_pixelwidth minimum is 1.0\n";
@@ -475,8 +526,10 @@ void interfaceImpl_t::render(paramMap_t &params)
 		WARNING << "'tolerance' parameter obsolete, use 'AA_threshold' instead\n";
 	else
 		params.getParam("AA_threshold", AA_threshold);
-	// single pass maximum samples value
-	params.getParam("AA_onepass_max", AA_onepass_max);
+	bool AA_jitterfirst = false;
+	params.getParam("AA_jitterfirst", AA_jitterfirst);
+	bool clamp_rgb = false;
+	params.getParam("clamp_rgb", clamp_rgb);
 
 	if(*camera=="")
 	{
@@ -489,22 +542,9 @@ void interfaceImpl_t::render(paramMap_t &params)
 		return;
 	}
 
-	// indirect sampling parameters
-	int indir_sam = 0;
-	params.getParam("indirect_samples", indir_sam);
-	int g=int(sqrt((float)indir_sam));
-	g *= g;
-	if (g!=indir_sam) {
-		// must be integer squared value
-		WARNING<< "Indirect samples value changed from " << indir_sam << " to " << g << endl;
-		indir_sam = g;
-	}
-	PFLOAT indir_pow = 1;
-	params.getParam("indirect_power", indir_pow);
-
 	// exposure and gamma adjustment controls
 	// when exposure set to 0, it is disabled, gamma=1 is no change
-	CFLOAT exposure=sqrt(2.0), gamma=1.0;
+	CFLOAT exposure=0, gamma=1;
 	params.getParam("exposure", exposure);
 	params.getParam("gamma", gamma);
 
@@ -515,7 +555,12 @@ void interfaceImpl_t::render(paramMap_t &params)
 	color_t fgcol(1.0);
 	params.getParam("fog_color", fgcol);
 
+
+#if HAVE_PTHREAD
+	scene_t *pscene=threadedscene_t::factory();
+#else
 	scene_t *pscene=scene_t::factory();
+#endif
 	scene_t &scene=*pscene;
 
 	camera_t *cam=camera_table[*camera];
@@ -531,16 +576,26 @@ void interfaceImpl_t::render(paramMap_t &params)
 			i!=filter_table.end();++i)
 		scene.addFilter((*i).second);
 
-	outTga_t out(cam->resX(), cam->resY(), outfile->c_str(), save_alpha);
+	// output filetype, types=tga/hdr/exr, default = tga
+	string _output_type="tga";
+	const string *output_type=&_output_type;
+	params.getParam("output_type", output_type);
+	// for exr output, extra flags to set save (half|float) and compression type
+	string _exr_flags="";	// defaults half, zip, no zbuf
+	const string *exr_flags=&_exr_flags;
+	params.getParam("exr_flags", exr_flags);
 
-	if (indir_sam)
-		cout << "Rendering with " << indir_sam << " indirect samples and "
-			<< raydepth << " raydepth\n";
-	else cout << "Rendering with " << raydepth << " raydepth\n";
-	if (AA_onepass_max)
-		cout << "Single pass anti-aliasing, using " << AA_minsamples << " minimum and "
-		     << AA_onepass_max << " maximum samples per pixel.\n";
-	else if (AA_passes)
+	// alpha premultiply, off by default
+	string _alpha_premul="off";
+	const string *alpha_premul=&_alpha_premul;
+	params.getParam("alpha_premultiply", alpha_premul);
+	// alpha mask background, off by default
+	string _alpha_mbg="off";
+	const string *alpha_mbg=&_alpha_mbg;
+	params.getParam("alpha_maskbackground", alpha_mbg);
+
+	cout << "Rendering with " << raydepth << " raydepth\n";
+	if (AA_passes)
 		cout << AA_passes << " anti-alias passes and "
 			<< AA_minsamples << " minimum samples per pass, "
 			<< AA_passes*AA_minsamples << " samples total.\n";
@@ -556,8 +611,17 @@ void interfaceImpl_t::render(paramMap_t &params)
 			scene.setBackground(background_table[*background]);
 	}
 
-	// indirect values
-	scene.setIndirect(indir_sam, indir_pow);
+	// alpha premultiply flag
+	if (*alpha_premul=="on")
+		scene.alphaPremultiply(true);
+	else
+		scene.alphaPremultiply(false);
+	// alpha background mask
+	if (*alpha_mbg=="on")
+		scene.alphaMaskBackground(true);
+	else
+		scene.alphaMaskBackground(false);
+
 	// gamma & exposure
 	scene.setExposure(exposure);
 	scene.setGamma(gamma);
@@ -565,15 +629,41 @@ void interfaceImpl_t::render(paramMap_t &params)
 	scene.setFog(fgdens, fgcol);
 
 	// set the AA params
-	scene.setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_onepass_max);
-
+	scene.setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_jitterfirst);
+	scene.clampRGB(clamp_rgb);
+	scene.setRegion(xmin,xmax,ymin,ymax);
 	scene.setBias(bias);
 	if(cachedPathLight) scene.setRepeatFirst();
 
-        scene.setCPUs(cpus);
-        scene.render(out);
+	scene.setCPUs(cpus);
 
-	out.flush();
+	// tone mapping bypassed when hdr/exr output is requested
+	if (*output_type=="hdr") {
+		outHDR_t hdrout(cam->resX(), cam->resY(), outfile->c_str());
+		scene.tonemap(false);
+		scene.render(hdrout);
+		hdrout.flush();
+	}
+	else if (int((*output_type).find("exr"))!=-1) {
+#if HAVE_EXR
+		outEXR_t exrout(cam->resX(), cam->resY(), outfile->c_str(), *exr_flags);
+		scene.tonemap(false);
+		scene.render(exrout);
+		exrout.flush();
+#else
+		cout << "Yafray was compiled without OpenEXR support.\nImage saved in hdr format instead." << endl;
+		outHDR_t hdrout(cam->resX(), cam->resY(), outfile->c_str());
+		scene.tonemap(false);
+		scene.render(hdrout);
+		hdrout.flush();
+#endif
+	}
+	else {
+		outTga_t tgaout(cam->resX(), cam->resY(), outfile->c_str(), save_alpha);
+		scene.tonemap(true);
+		scene.render(tgaout);
+		tgaout.flush();
+	}
 
 	delete pscene;
 }
@@ -583,12 +673,17 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 	string _camera,_background;
 	const string *camera=&_camera,*background=&_background;
 	int raydepth = 3;
+	GFLOAT xmin=-1.5,xmax=1.5,ymin=-1.5,ymax=1.5;
 	PFLOAT bias = 0.1;
 
 	params.getParam("camera_name", camera);
 	params.getParam("raydepth", raydepth);
 	params.getParam("bias", bias);
 	params.getParam("background_name", background);
+	params.getParam("border_xmin",xmin);
+	params.getParam("border_xmax",xmax);
+	params.getParam("border_ymin",ymin);
+	params.getParam("border_ymax",ymax);
 
 	// get new AA parameters
 	int AA_passes=0, AA_minsamples=1;
@@ -607,8 +702,7 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 		INFO << "AA_minsamples must be at least 1\n";
 		AA_minsamples = 1;
 	}
-	PFLOAT AA_pixelwidth=1.0, AA_threshold=0.125;
-	int AA_onepass_max = 0;
+	PFLOAT AA_pixelwidth=1.5, AA_threshold=0.05;
 	params.getParam("AA_pixelwidth", AA_pixelwidth);
 	if (AA_pixelwidth<1.0) {
 	  INFO << "AA_pixelwidth minimum is 1.0\n";
@@ -622,8 +716,10 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 		WARNING << "'tolerance' parameter obsolete, use 'AA_threshold' instead\n";
 	else
 		params.getParam("AA_threshold", AA_threshold);
-	// single pass maximum samples value
-	params.getParam("AA_onepass_max", AA_onepass_max);
+	bool AA_jitterfirst = false;
+	params.getParam("AA_jitterfirst", AA_jitterfirst);
+	bool clamp_rgb = false;
+	params.getParam("clamp_rgb", clamp_rgb);
 
 	if(*camera=="")
 	{
@@ -636,22 +732,9 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 		return;
 	}
 
-	// indirect sampling parameters
-	int indir_sam = 0;
-	params.getParam("indirect_samples", indir_sam);
-	int g=int(sqrt((float)indir_sam));
-	g *= g;
-	if (g!=indir_sam) {
-		// must be integer squared value
-		WARNING<< "Indirect samples value changed from " << indir_sam << " to " << g << endl;
-		indir_sam = g;
-	}
-	PFLOAT indir_pow = 1;
-	params.getParam("indirect_power", indir_pow);
-
 	// exposure and gamma adjustment controls
 	// when exposure set to 0, it is disabled, gamma=1 is no change
-	CFLOAT exposure=sqrt(2.0), gamma=1.0;
+	CFLOAT exposure=0, gamma=1;
 	params.getParam("exposure", exposure);
 	params.getParam("gamma", gamma);
 
@@ -662,7 +745,11 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 	color_t fgcol(1.0);
 	params.getParam("fog_color", fgcol);
 
+#if HAVE_PTHREAD
+	scene_t *pscene=threadedscene_t::factory();
+#else
 	scene_t *pscene=scene_t::factory();
+#endif
 	scene_t &scene=*pscene;
 	camera_t *cam=camera_table[*camera];
 
@@ -677,15 +764,17 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 			i!=filter_table.end();++i)
 		scene.addFilter((*i).second);
 
+	// alpha premultiply, off by default
+	string _alpha_premul="off";
+	const string *alpha_premul=&_alpha_premul;
+	params.getParam("alpha_premultiply", alpha_premul);
+	// alpha mask background, off by default
+	string _alpha_mbg="off";
+	const string *alpha_mbg=&_alpha_mbg;
+	params.getParam("alpha_maskbackground", alpha_mbg);
 
-	if (indir_sam)
-		cout << "Rendering with " << indir_sam << " indirect samples and "
-			<< raydepth << " raydepth\n";
-	else cout << "Rendering with " << raydepth << " raydepth\n";
-	if (AA_onepass_max)
-		cout << "Single pass anti-aliasing, using " << AA_minsamples << " minimum and "
-		     << AA_onepass_max << " maximum samples per pixel.\n";
-	else if (AA_passes)
+	cout << "Rendering with " << raydepth << " raydepth\n";
+	if (AA_passes)
 		cout << AA_passes << " anti-alias passes and "
 			<< AA_minsamples << " minimum samples per pass, "
 			<< AA_passes*AA_minsamples << " samples total.\n";
@@ -701,22 +790,36 @@ void interfaceImpl_t::render(paramMap_t &params,colorOutput_t &output)
 			scene.setBackground(background_table[*background]);
 	}
 
-	// indirect values
-	scene.setIndirect(indir_sam, indir_pow);
+	// alpha premultiply flag
+	if (*alpha_premul=="on")
+		scene.alphaPremultiply(true);
+	else
+		scene.alphaPremultiply(false);
+	// alpha background mask
+	if (*alpha_mbg=="on")
+		scene.alphaMaskBackground(true);
+	else
+		scene.alphaMaskBackground(false);
+
 	// gamma & exposure
 	scene.setExposure(exposure);
 	scene.setGamma(gamma);
+	// forgot this part, for external render, always set tonemap to true,
+	// user can still bypass it by setting gamma=1, exposure=0
+	scene.tonemap(true);
+	
 	// fog
 	scene.setFog(fgdens, fgcol);
 
 	// set the AA params
-	scene.setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_onepass_max);
-
+	scene.setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_jitterfirst);
+	scene.clampRGB(clamp_rgb);
+	scene.setRegion(xmin,xmax,ymin,ymax);
 	scene.setBias(bias);
 	if(cachedPathLight) scene.setRepeatFirst();
 
-        scene.setCPUs(cpus);
-        scene.render(output);
+	scene.setCPUs(cpus);
+	scene.render(output);
 
 	output.flush();
 }
