@@ -313,7 +313,7 @@ int main(int argc,char **argv)
 		scene_strat = render_t::THREAD;
 
 	// FORCE
-	scene_strat = render_t::MONO;
+	//scene_strat = render_t::MONO;
 
 	if(sintax.parse())
 	{
