@@ -27,6 +27,9 @@
 #include "threadedscene.h"
 #include "forkedscene.h"
 
+#include "targaIO.h"
+#include "HDR_io.h"
+#include "EXR_io.h"
 
 #define WARNING cerr<<"[Warning]: "
 #define ERRORMSG cerr<<"[Error]: "
@@ -41,7 +44,7 @@ render_t::render_t(int ncpus, strategy_t strat, const string &pluginpath):M(1.0)
 	for(int i=0;i<MAX_AST;++i)
 		handler[i]=NULL;
 	cpus=ncpus;
-        strategy = strat;
+	strategy = strat;
 	scymin=scxmin=-2;
 	scymax=scxmax=2;
 	handler[AST_LITEM]=&render_t::itemList;
@@ -681,18 +684,21 @@ void * render_t::object(ast_t *ast)
 		WARNING<<"Object "<<object->name<<" redefined\n";
 		delete object_table[object->name];
 	}
-	obj->castShadows(object->shadow);
-	obj->useForRadiosity(object->emit_rad);
-	obj->reciveRadiosity(object->recv_rad);
-	obj->caustics(object->caustics);
-	color_t rcolor(object->caus_rcolor.r,
-			object->caus_rcolor.g,
-			object->caus_rcolor.b);
-	color_t tcolor(object->caus_tcolor.r,
-			object->caus_tcolor.g,
-			object->caus_tcolor.b);
-	obj->setCaustic(rcolor,tcolor,object->caus_IOR);
-	obj->setShader(shader_table[object->shader]);
+	// error was here, don't reset vars when object is reference, already set from original object
+	if (!isref) {
+		obj->castShadows(object->shadow);
+		obj->useForRadiosity(object->emit_rad);
+		obj->reciveRadiosity(object->recv_rad);
+		obj->caustics(object->caustics);
+		color_t rcolor(object->caus_rcolor.r,
+				object->caus_rcolor.g,
+				object->caus_rcolor.b);
+		color_t tcolor(object->caus_tcolor.r,
+				object->caus_tcolor.g,
+				object->caus_tcolor.b);
+		obj->setCaustic(rcolor, tcolor, object->caus_IOR);
+		obj->setShader(shader_table[object->shader]);
+	}
 	object_table[object->name]=obj;
 	INFO<<"Added object "<<object->name<<endl;
 	return obj;
@@ -715,21 +721,21 @@ void * render_t::mesh(ast_t *ast)
 	}
 	for(vector<triangle_t>::iterator i=faces.begin();i!=faces.end();++i)
 	{
-		int n=(int)((*i).a);
-		if((n>=(int)mesh->points->points.size()) || (n<0))
+		long int n=(int)((*i).a);
+		if((n>=(long int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
 		(*i).a=&(mesh->points->points)[n];
 		n=(int)((*i).b);
-		if((n>=(int)mesh->points->points.size()) || (n<0))
+		if((n>=(long int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
 		(*i).b=&(mesh->points->points)[n];
-		n=(int)((*i).c);
-		if((n>=(int)mesh->points->points.size()) || (n<0))
+		n=(long int)((*i).c);
+		if((n>=(long int)mesh->points->points.size()) || (n<0))
 			{ WARNING<<"Point "<<n<<" out of bounds in object\n"; n=0; }
 		(*i).c=&(mesh->points->points)[n];
 		
 		(*i).recNormal();
-		int ishader=(int)(*i).getShader();
+		long int ishader=(long int)(*i).getShader();
 		if(ishader<0) 
 			(*i).setShader(NULL);
 		else
@@ -750,13 +756,12 @@ void * render_t::mesh(ast_t *ast)
 		}
 	}
 	vector<vector3d_t> normals;
-	meshObject_t *obj = meshObject_t::factory(mesh->points->points, normals,
+	meshObject_t *obj = meshObject_t::factory(mesh->orco, M, mesh->points->points, normals,
 						mesh->faces->faces, mesh->faces->facesuv, mesh->faces->faces_vcol);
 
-	obj->hasOrco(mesh->orco);
-	obj->transform(M);
-	if(mesh->autosmooth)
-		obj->autoSmooth(mesh->angle);
+	if(mesh->autosmooth) obj->autoSmooth(mesh->angle);
+	obj->tangentsFromUV();
+	
 	return obj;
 }
 
@@ -812,11 +817,38 @@ void * render_t::camera(ast_t *ast)
 {
 	camera_data_t *camera=(camera_data_t *)ast;
 	if (camera->name=="") return NULL;
-	cameraType ct = CAM_PERSPECTIVE;
-	if (camera->camtype=="ortho") ct = CAM_ORTHO;
+	// camera type
+	camera_t::cameraType ct = camera_t::CM_PERSPECTIVE;
+	if (camera->camtype=="ortho")
+		ct = camera_t::CM_ORTHO;
+	else if (camera->camtype=="spherical")
+		ct = camera_t::CM_SPHERICAL;
+	else if (camera->camtype=="lightprobe")
+		ct = camera_t::CM_LIGHTPROBE;
+	// bokeh type
+	camera_t::bokehType bt = camera_t::BK_DISK1;
+	if (camera->bkhtype=="disk2")
+		bt = camera_t::BK_DISK2;
+	else if (camera->bkhtype=="triangle")
+		bt = camera_t::BK_TRI;
+	else if (camera->bkhtype=="square")
+		bt = camera_t::BK_SQR;
+	else if (camera->bkhtype=="pentagon")
+		bt = camera_t::BK_PENTA;
+	else if (camera->bkhtype=="hexagon")
+		bt = camera_t::BK_HEXA;
+	else if (camera->bkhtype=="ring")
+		bt = camera_t::BK_RING;
+	// bokeh bias
+	camera_t::bkhBiasType bbt = camera_t::BB_NONE;
+	if (camera->bkhbias=="center")
+		bbt = camera_t::BB_CENTER;
+	else if (camera->bkhbias=="edge")
+		bbt = camera_t::BB_EDGE;
 	camera_t *cam=new camera_t(camera->from, camera->to, camera->up,
-														camera->resx, camera->resy,
-														camera->dfocal, camera->laperture, camera->dofdist, camera->dof_use_qmc, ct);
+														camera->resx, camera->resy, camera->aspect, camera->dfocal,
+														camera->laperture, camera->dofdist, camera->dof_use_qmc,
+														ct, bt, bbt, camera->bkh_ro);
 	if (camera_table.find(camera->name)!=camera_table.end())
 	{
 		WARNING << "Camera " << camera->name << " redefined\n";
@@ -1051,8 +1083,7 @@ void * render_t::render(ast_t *ast)
 		INFO << "AA_minsamples must be at least 1\n";
 		AA_minsamples = 1;
 	}
-	PFLOAT AA_pixelwidth=1.0, AA_threshold=0.125;
-	int AA_onepass_max = 0;
+	PFLOAT AA_pixelwidth=1.5, AA_threshold=0.05;
 	params.getParam("AA_pixelwidth", AA_pixelwidth);
 	if (AA_pixelwidth<1.0) {
 	  INFO << "AA_pixelwidth minimum is 1.0\n";
@@ -1066,8 +1097,10 @@ void * render_t::render(ast_t *ast)
 		WARNING << "'tolerance' parameter obsolete, use 'AA_threshold' instead\n";
 	else
 		params.getParam("AA_threshold", AA_threshold);
-	// single pass maximum samples value
-	params.getParam("AA_onepass_max", AA_onepass_max);
+	bool AA_jitterfirst = false;
+	params.getParam("AA_jitterfirst", AA_jitterfirst);
+	bool clamp_rgb = false;
+	params.getParam("clamp_rgb", clamp_rgb);
 
 	if(*camera=="")
 	{
@@ -1080,22 +1113,9 @@ void * render_t::render(ast_t *ast)
 		return NULL;
 	}
 
-	// indirect sampling parameters
-	int indir_sam = 0;
-	params.getParam("indirect_samples", indir_sam);
-	int g=int(sqrt((float)indir_sam));
-	g *= g;
-	if (g!=indir_sam) {
-		// must be integer squared value
-		WARNING<< "Indirect samples value changed from " << indir_sam << " to " << g << endl;
-		indir_sam = g;
-	}
-	PFLOAT indir_pow = 1;
-	params.getParam("indirect_power", indir_pow);
-
 	// exposure and gamma adjustment controls
 	// when exposure set to 0, it is disabled, gamma=1 is no change
-	CFLOAT exposure=sqrt(2.0), gamma=1.0;
+	CFLOAT exposure=0, gamma=1;
 	params.getParam("exposure", exposure);
 	params.getParam("gamma", gamma);
 
@@ -1111,10 +1131,16 @@ void * render_t::render(ast_t *ast)
 		case MONO:
 			scene = scene_t::factory();
 			break;
+#if HAVE_PTHREAD
 		case THREAD:
+#if HAVE_PTHREAD
+			scene = threadedscene_t::factory();
+#else
 			scene = scene_t::factory();
+#endif
 			//scene = new threadedscene_t;
 			break;
+#endif
 		case FORK:
 			scene = scene_t::factory();
 			//scene = new forkedscene_t;
@@ -1125,64 +1151,111 @@ void * render_t::render(ast_t *ast)
 	}
 
 	camera_t *cam=camera_table[*camera];
+	scene->setCamera(cam);
 
-        scene->setCamera(cam);
 	for(map<string,object3d_t *>::iterator i=object_table.begin();
 			i!=object_table.end();++i)
-                scene->addObject((*i).second);
+		scene->addObject((*i).second);
+
 	for(map<string,light_t *>::iterator i=light_table.begin();
 			i!=light_table.end();++i)
-                scene->addLight((*i).second);
+		scene->addLight((*i).second);
+
 	for(map<string,filter_t *>::iterator i=filter_table.begin();
 			i!=filter_table.end();++i)
-                scene->addFilter((*i).second);
+		scene->addFilter((*i).second);
 
-	outTga_t out(cam->resX(), cam->resY(), outfile->c_str(), save_alpha);
+	// output filetype, types=tga/hdr/exr, default = tga
+	string _output_type="tga";
+	const string *output_type=&_output_type;
+	params.getParam("output_type", output_type);
+	// for exr output, extra flags to set save (half|float) and compression type
+	string _exr_flags="";	// defaults half, zip, no zbuf
+	const string *exr_flags=&_exr_flags;
+	params.getParam("exr_flags", exr_flags);
 
-	if (indir_sam)
-		cout << "Rendering with " << indir_sam << " indirect samples and "
-			<< raydepth << " raydepth\n";
-	else cout << "Rendering with " << raydepth << " raydepth\n";
-	if (AA_onepass_max)
-		cout << "Single pass anti-aliasing, using " << AA_minsamples << " minimum and "
-		     << AA_onepass_max << " maximum samples per pixel.\n";
-	else if (AA_passes)
+	// alpha premultiply, off by default
+	string _alpha_premul="off";
+	const string *alpha_premul=&_alpha_premul;
+	params.getParam("alpha_premultiply", alpha_premul);
+	// alpha mask background, off by default
+	string _alpha_mbg="off";
+	const string *alpha_mbg=&_alpha_mbg;
+	params.getParam("alpha_maskbackground", alpha_mbg);
+
+	cout << "Rendering with " << raydepth << " raydepth\n";
+	if (AA_passes)
 		cout << AA_passes << " anti-alias passes and "
 			<< AA_minsamples << " minimum samples per pass, "
 			<< AA_passes*AA_minsamples << " samples total.\n";
 	else cout << "No anti-aliasing.\n";
 
-        scene->setMaxRayDepth(raydepth);
+	scene->setMaxRayDepth(raydepth);
 
 	if(*background!="")
 	{
 		if(background_table.find(*background)==background_table.end())
 			WARNING<<"Background "<<*background<<" does not exist\n";
 		else
-                        scene->setBackground(background_table[*background]);
+			scene->setBackground(background_table[*background]);
 	}
 
-	// indirect values
-        scene->setIndirect(indir_sam, indir_pow);
+	// alpha premultiply
+	if (*alpha_premul=="on")
+		scene->alphaPremultiply(true);
+	else
+		scene->alphaPremultiply(false);
+	// alpha background
+	if (*alpha_mbg=="on")
+		scene->alphaMaskBackground(true);
+	else
+		scene->alphaMaskBackground(false);
+
 	// gamma & exposure
-        scene->setExposure(exposure);
-        scene->setGamma(gamma);
+	scene->setExposure(exposure);
+	scene->setGamma(gamma);
 	// fog
-        scene->setFog(fgdens, fgcol);
+	scene->setFog(fgdens, fgcol);
 
 	// set the AA params
-        scene->setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_onepass_max);
+	scene->setAASamples(AA_passes, AA_minsamples, AA_pixelwidth, AA_threshold, AA_jitterfirst);
+	scene->clampRGB(clamp_rgb);
 
-        scene->setBias(bias);
-        if(cachedPathLight) scene->setRepeatFirst();
+	scene->setBias(bias);
+	if(cachedPathLight) scene->setRepeatFirst();
 
-        scene->setRegion(scxmin,scxmax,scymin,scymax);
-        scene->setCPUs(cpus);
-        scene->render(out);
+	scene->setRegion(scxmin,scxmax,scymin,scymax);
+	scene->setCPUs(cpus);
 
-	out.flush();
+	// tone mapping bypassed when hdr/exr output is requested
+	if (*output_type=="hdr") {
+		outHDR_t hdrout(cam->resX(), cam->resY(), outfile->c_str());
+		scene->tonemap(false);
+		scene->render(hdrout);
+		hdrout.flush();
+	}
+	else if (int((*output_type).find("exr"))!=-1) {
+#if HAVE_EXR
+		outEXR_t exrout(cam->resX(), cam->resY(), outfile->c_str(), *exr_flags);
+		scene->tonemap(false);
+		scene->render(exrout);
+		exrout.flush();
+#else
+		cout << "Yafray was compiled without OpenEXR support.\nImage saved in hdr format instead." << endl;
+		outHDR_t hdrout(cam->resX(), cam->resY(), outfile->c_str());
+		scene->tonemap(false);
+		scene->render(hdrout);
+		hdrout.flush();
+#endif
+	}
+	else {
+		outTga_t tgaout(cam->resX(), cam->resY(), outfile->c_str(), save_alpha);
+		scene->tonemap(true);
+		scene->render(tgaout);
+		tgaout.flush();
+	}
 
-        delete scene;
+	delete scene;
 
 	return NULL;
 }
