@@ -408,7 +408,25 @@ gObjectIterator_t<T,D,CROSS>::gObjectIterator_t(const gBoundTreeNode_t<T> *r,con
 	{
 		currT=currN->begin();
 		currTend=currN->end();
-		if(currT==currTend) ++(*this);
+		//if(currT==currTend) ++(*this);
+		while(currT==currTend)
+		{
+			bool first=true;
+			while(first || !(currN->isLeaf()))
+			{
+				first=false;
+				upFirstRight();
+				if(currN==NULL)
+				{
+					end=true;
+					return;
+				}
+				DOWN_RIGHT(currN);
+				downLeft();
+			}
+			currT=currN->begin();
+			currTend=currN->end();
+		}
 	}
 }
 
