@@ -34,9 +34,9 @@ void operator >> (unsigned char *data,color_t &c)
 
 void operator << (unsigned char *data,const color_t &c)
 {
-	data[0]=(char)( ((CFLOAT)255) * ((c.R>(CFLOAT)1) ? 1 :c.R) );
-	data[1]=(char)( ((CFLOAT)255) * ((c.G>(CFLOAT)1) ? 1 :c.G) );
-	data[2]=(char)( ((CFLOAT)255) * ((c.B>(CFLOAT)1) ? 1 :c.B) );
+	data[0]=(char) (((c.R<(CFLOAT)0) ? 0 : ((c.R>(CFLOAT)1) ? 255 : (((CFLOAT)255)*c.R) )) );
+	data[1]=(char) (((c.G<(CFLOAT)0) ? 0 : ((c.G>(CFLOAT)1) ? 255 : (((CFLOAT)255)*c.G) )) );
+	data[2]=(char) (((c.B<(CFLOAT)0) ? 0 : ((c.B>(CFLOAT)1) ? 255 : (((CFLOAT)255)*c.B) )) );
 }
 
 void operator >> (unsigned char *data,colorA_t &c)
@@ -50,10 +50,10 @@ void operator >> (unsigned char *data,colorA_t &c)
 
 void operator << (unsigned char *data,const colorA_t &c)
 {
-	data[0] = (char)( ((CFLOAT)255) * ((c.R>(CFLOAT)1) ? 1 :c.R) );
-	data[1] = (char)( ((CFLOAT)255) * ((c.G>(CFLOAT)1) ? 1 :c.G) );
-	data[2] = (char)( ((CFLOAT)255) * ((c.B>(CFLOAT)1) ? 1 :c.B) );
-	data[3] = (char)( ((CFLOAT)255) * ((c.A>(CFLOAT)1) ? 1 :c.A) );
+	data[0]=(char) (((c.R<(CFLOAT)0) ? 0 : ((c.R>(CFLOAT)1) ? 255 : (((CFLOAT)255)*c.R) )) );
+	data[1]=(char) (((c.G<(CFLOAT)0) ? 0 : ((c.G>(CFLOAT)1) ? 255 : (((CFLOAT)255)*c.G) )) );
+	data[2]=(char) (((c.B<(CFLOAT)0) ? 0 : ((c.B>(CFLOAT)1) ? 255 : (((CFLOAT)255)*c.B) )) );
+	data[3]=(char) (((c.A<(CFLOAT)0) ? 0 : ((c.A>(CFLOAT)1) ? 255 : (((CFLOAT)255)*c.A) )) );
 }
 
 void operator >> (float *data,color_t &c)
@@ -88,8 +88,6 @@ void operator << (float *data,const colorA_t &c)
 	data[3] = c.A;
 }
 
-//char lumtochar[12]=" -;+&%G@$*";
-
 ostream & operator << (ostream & out,const color_t c)
 {
 	out<<"["<<c.R<<" "<<c.G<<" "<<c.B<<"]";
@@ -139,48 +137,21 @@ color_t convergenceAccell(const color_t &cn_1,const color_t &cn0,const color_t &
 	return color_t(r,g,b);
 }
 
-color_t BilerpWeight(CFLOAT xf, CFLOAT yf,
-		const color_t &c1, const color_t &c2,
-		const color_t &c3, const color_t &c4)
-
-{
-	CFLOAT dx=xf-floor(xf), dy=yf-floor(yf);
-	CFLOAT w0=(1-dx)*(1-dy), w1=(1-dx)*dy, w2=dx*(1-dy), w3=dx*dy;
-	return color_t(	w0*c1.R + w1*c3.R + w2*c2.R + w3*c4.R,
-			w0*c1.G + w1*c3.G + w2*c2.G + w3*c4.G,
-			w0*c1.B + w1*c3.B + w2*c2.B + w3*c4.B);
-}
-
-colorA_t BilerpWeight(CFLOAT xf, CFLOAT yf,
-		const colorA_t &c1, const colorA_t &c2,
-		const colorA_t &c3, const colorA_t &c4)
-{
-	CFLOAT dx=xf-floor(xf), dy=yf-floor(yf);
-	CFLOAT w0=(1-dx)*(1-dy), w1=(1-dx)*dy, w2=dx*(1-dy), w3=dx*dy;
-	return colorA_t(	w0*c1.R + w1*c3.R + w2*c2.R + w3*c4.R,
-									w0*c1.G + w1*c3.G + w2*c2.G + w3*c4.G,
-									w0*c1.B + w1*c3.B + w2*c2.B + w3*c4.B,
-									w0*c1.A + w1*c3.A + w2*c2.A + w3*c4.A);
-}
-
 rgbe_t::rgbe_t(const color_t &s)
 {
-  CFLOAT v;
-  int e;
-
-  v = s.getRed();
-  if (s.getGreen() > v) v = s.getGreen();
-  if (s.getBlue() > v) v = s.getBlue();
-  if (v < 1e-32) {
-    rgbe[0] = rgbe[1] = rgbe[2] = rgbe[3] = 0;
-  }
-  else {
-    v = frexp(v,&e) * 256.0/v;
-    rgbe[0] = (unsigned char) (s.getRed() * v);
-    rgbe[1] = (unsigned char) (s.getGreen() * v);
-    rgbe[2] = (unsigned char) (s.getBlue() * v);
-    rgbe[3] = (unsigned char) (e + 128);
-  }
+	CFLOAT v = s.getR();
+	if (s.getG() > v) v = s.getG();
+	if (s.getB() > v) v = s.getB();
+	if (v<1e-32f)
+		rgbe[0] = rgbe[1] = rgbe[2] = rgbe[3] = 0;
+	else {
+		int e;
+		v = frexp(v, &e) * 256.0/v;
+		rgbe[0] = (unsigned char) (s.getR() * v);
+		rgbe[1] = (unsigned char) (s.getG() * v);
+		rgbe[2] = (unsigned char) (s.getB() * v);
+		rgbe[3] = (unsigned char) (e + 128);
+	}
 }
 
 __END_YAFRAY
