@@ -113,7 +113,7 @@ inline T * convertPointer(vector<T> &d,const vector<T> &v,const T *p)
 	return (T *)((p-&v[0])+&d[0]);
 }
 
-tnode_t * buildTriangleTree(std::vector<triangle_t*> *v,
+tnode_t * buildTriangleTree(std::vector<triangle_t*> *v, unsigned int maxdepth,
 		const bound_t &bound,unsigned int dtol=1,unsigned int depth=1,unsigned int lostd=0,
 		PFLOAT &avgdepth=foo1, PFLOAT &avgsize=foo2);
 
@@ -152,24 +152,25 @@ meshObject_t::meshObject_t(const vector<point3d_t> &ver, const vector<vector3d_t
 		(*ltri)[i-triangles.begin()]=&(*i);
 
 	//tree=buildGenericTree(ltri,face_calc_bound,face_is_in_bound,face_get_pos,10);
-	tree=buildTriangleTree(ltri,face_calc_bound(*ltri),4);
+	unsigned int maxdepth = (unsigned int)(8.0 + 1.8755035531556525*log((PFLOAT)triangles.size()));
+	tree=buildTriangleTree(ltri, maxdepth, face_calc_bound(*ltri),4);
 }
 
-meshObject_t::meshObject_t(const matrix4x4_t &M, const vector<point3d_t> &ver, const vector<vector3d_t> &nor,
+meshObject_t::meshObject_t(bool _hasorco, const matrix4x4_t &M, const vector<point3d_t> &ver, const vector<vector3d_t> &nor,
 				const vector<triangle_t> &ts, const vector<GFLOAT> &fuv, const vector<CFLOAT> &fvcol)
 {
-	vertices=ver;
-	normals=nor;
-	triangles=ts;
-	unt=true;
-	hasorco=false;
-	if ( (ver.empty()) || (ts.empty()))
-		cout<<"Error null mesh\n";
-	shader=NULL;
+	hasorco = _hasorco;
+	vertices = ver;
+	normals = nor;
+	triangles = ts;
+	unt = true;
+	if ((ver.empty()) || (ts.empty()))
+		cout << "Error null mesh\n";
+	shader = NULL;
 	//if(ver!=NULL) recalcBound();
 	facesuv = fuv;
 	faces_vcol = fvcol;
-	
+
 	for(vector<triangle_t>::iterator i=triangles.begin();i!=triangles.end();++i)
 	{
 		i->a=convertPointer(vertices,ver,i->a);
@@ -191,65 +192,144 @@ meshObject_t::meshObject_t(const matrix4x4_t &M, const vector<point3d_t> &ver, c
 
 void meshObject_t::autoSmooth(PFLOAT angle)
 {
-	vector<list<triangle_t * > > access(vertices.size());
-	int ia,ib,ic;
-	for(vector<triangle_t>::iterator ite=triangles.begin();ite!=triangles.end()
-			;ite++)
+	// if no smoothing needed, normal equal to geometric normal,
+	// is returned automatically in surfacepoint, no calculation needed
+	if (angle<1) return;
+	unsigned int i1, i2, i3;
+	vector<triangle_t>::iterator tri;
+	// if everything is smoothed, only need as many normals as there are vertices
+	if (angle>=180)
 	{
-		ia=(*ite).a-(&vertices[0]);
-		ib=(*ite).b-(&vertices[0]);
-		ic=(*ite).c-(&vertices[0]);
-		access[ia].push_back(&(*ite));
-		access[ib].push_back(&(*ite));
-		access[ic].push_back(&(*ite));
-	}
-	//if (normals.size()) normals.clear();
-	normals.resize(triangles.size()*3);
-
-	PFLOAT cosang;
-	if (angle<0) cosang=-2;
-	else cosang=cos(angle* (((PFLOAT)2)*M_PI)/((PFLOAT)360));
-	vector3d_t norm,na,nb,nc;
-	int itri=0;
-	for(vector<triangle_t>::iterator ite=triangles.begin();ite!=triangles.end()
-			;ite++)
-	{
-		norm=(*ite).N();
-		ia=(*ite).a-(&vertices[0]);
-		ib=(*ite).b-(&vertices[0]);
-		ic=(*ite).c-(&vertices[0]);
-		na=vector3d_t(0,0,0);
-		for(list<triangle_t *>::iterator ite2=access[ia].begin();
-				ite2!=access[ia].end();ite2++)
+		normals.resize(vertices.size());
+		for (tri=triangles.begin();tri!=triangles.end();tri++)
 		{
-			if( (((*ite2)->N())*norm)>cosang ) na=na+(*ite2)->N();
+			i1 = tri->a - (&vertices[0]);
+			i2 = tri->b - (&vertices[0]);
+			i3 = tri->c - (&vertices[0]);
+			normals[i1] += tri->N();
+			normals[i2] += tri->N();
+			normals[i3] += tri->N();
+			tri->na = &normals[i1];
+			tri->nb = &normals[i2];
+			tri->nc = &normals[i3];
 		}
-		na.normalize();
-		normals[itri]=na;
-		(*ite).na=&normals[itri];
-
-		nb=vector3d_t(0,0,0);
-		for(list<triangle_t *>::iterator ite2=access[ib].begin();
-				ite2!=access[ib].end();ite2++)
-		{
-			if( (((*ite2)->N())*norm)>cosang ) nb=nb+(*ite2)->N();
-		}
-		nb.normalize();
-		normals[itri+1]=nb;
-		(*ite).nb=&normals[itri+1];
-
-		nc=vector3d_t(0,0,0);
-		for(list<triangle_t *>::iterator ite2=access[ic].begin();
-				ite2!=access[ic].end();ite2++)
-		{
-			if( (((*ite2)->N())*norm)>cosang ) nc=nc+(*ite2)->N();
-		}
-		nc.normalize();
-		normals[itri+2]=nc;
-		(*ite).nc=&normals[itri+2];
-		itri+=3;
+		for (i1=0;i1<normals.size();i1++)
+			normals[i1].normalize();
+		return;
 	}
 
+	// angle dependant smoothing
+	PFLOAT cosa = cos(angle*M_PI/180.0);
+	vector<vector<triangle_t*> > cnx(vertices.size());
+	for (tri=triangles.begin();tri!=triangles.end();tri++)
+	{
+		cnx[tri->a - (&vertices[0])].push_back(&(*tri));
+		cnx[tri->b - (&vertices[0])].push_back(&(*tri));
+		cnx[tri->c - (&vertices[0])].push_back(&(*tri));
+	}
+	vector<triangle_t*>::const_iterator vtri;
+
+	// Concerning memory usage, since depending on smoothing angle, normals could be shared,
+	// this could be further optimized to add only as many normals as needed.
+	// Similarly, faces which are not smoothed at all don't need vtxnorms so could be skipped.
+	// Array size of 3*number_of_triangles is really only the worst possible case.
+	// However, for a minimum storage triangle class (index only),
+	// for efficient access, it would probably be better to have a consistent layout.
+	// Another optimization is to only store normals in 2D form for all normals
+	// ie. only xy (or any other pair), z=sqrt(1-x^2-y^2) at runtime when needed, or in polar form (discard r),
+	// though both could be more overhead in terms of extra decoding time needed, first one less so probably.
+	normals.resize(3*triangles.size());
+	unsigned int idx=0;
+	for (tri=triangles.begin();tri!=triangles.end();tri++)
+	{
+		vector3d_t N = tri->N();
+		i1 = tri->a - (&vertices[0]);
+		i2 = tri->b - (&vertices[0]);
+		i3 = tri->c - (&vertices[0]);
+
+		vector3d_t sn(0, 0, 0);
+		for (vtri=cnx[i1].begin();vtri!=cnx[i1].end();++vtri)
+			if (((*vtri)->N()*N)>cosa) sn += (*vtri)->N();
+		sn.normalize();
+		normals[idx] = sn;
+		tri->na = &normals[idx];
+
+		sn.set(0, 0, 0);
+		for (vtri=cnx[i2].begin();vtri!=cnx[i2].end();++vtri)
+			if (((*vtri)->N()*N)>cosa) sn += (*vtri)->N();
+		sn.normalize();
+		normals[idx+1] = sn;
+		tri->nb = &normals[idx+1];
+
+		sn.set(0, 0, 0);
+		for (vtri=cnx[i3].begin();vtri!=cnx[i3].end();++vtri)
+			if (((*vtri)->N()*N)>cosa) sn += (*vtri)->N();
+		sn.normalize();
+		normals[idx+2] = sn;
+		tri->nc = &normals[idx+2];
+		idx += 3;
+	}
+}
+
+// tangents, derived from uv, if no uv, orco coords instead
+// call after autosmooth() and setting orco flag
+void meshObject_t::tangentsFromUV()
+{
+	bool hasuv = (!facesuv.empty());
+	if (!(hasuv || hasorco)) return;
+	tangents.resize(vertices.size());
+	vector3d_t sdir, tdir;
+	vector<triangle_t>::iterator tri;
+	for(tri=triangles.begin();tri!=triangles.end();tri++)
+	{
+		tri->ta = &tangents[tri->a - (&vertices[0])];
+		tri->tb = &tangents[tri->b - (&vertices[0])];
+		tri->tc = &tangents[tri->c - (&vertices[0])];
+	}
+	PFLOAT s1, s2, t1, t2;
+	unsigned int i0, i1, i2;
+	for(tri=triangles.begin();tri!=triangles.end();tri++)
+	{
+		if (hasuv) {
+			// from uv
+			if (tri->hasuv) {
+				s1 = tri->uv[2] - tri->uv[0];
+				s2 = tri->uv[4] - tri->uv[0];
+				t1 = tri->uv[3] - tri->uv[1];
+				t2 = tri->uv[5] - tri->uv[1];
+			}
+			else {
+				// no uv coords assigned to this tri
+				s1 = s2 = t1 = t2 = 0;
+			}
+		}
+		else {
+			// from orco
+			i0 = (tri->a - (&vertices[0]))+1;
+			i1 = (tri->b - (&vertices[0]))+1;
+			i2 = (tri->c - (&vertices[0]))+1;
+			s1 = 0.5*(vertices[i1].x - vertices[i0].x);
+			s2 = 0.5*(vertices[i2].x - vertices[i0].x);
+			t1 = 0.5*(vertices[i1].y - vertices[i0].y);
+			t2 = 0.5*(vertices[i2].y - vertices[i0].y);
+		}
+		PFLOAT r = s1*t2 - s2*t1;
+		if (r==0.0)
+			createCS(tri->N(), sdir, tdir);
+		else {
+			vector3d_t e1 = *(tri->b) - *(tri->a);
+			vector3d_t e2 = *(tri->c) - *(tri->a);
+			sdir = (t2*e1 - t1*e2)/r;
+			tdir = (s1*e2 - s2*e1)/r;
+			if (((sdir^tdir)*tri->N())<0.0) sdir *= -1.0;
+		}
+		*(tri->ta) += sdir;
+		*(tri->tb) += sdir;
+		*(tri->tc) += sdir;
+	}
+	// could adjust to make orthogonal, but not really needed for shading
+	for (unsigned int i=0;i<tangents.size();i++)
+		tangents[i].normalize();	
 }
 
 void meshObject_t::recalcBound()
@@ -385,7 +465,8 @@ void meshObject_t::transform(const matrix4x4_t &m)
 		(*ltri)[i-triangles.begin()]=&(*i);
 	if(tree!=NULL) delete tree;
 	//tree=buildGenericTree(ltri,face_calc_bound,face_is_in_bound,face_get_pos,4);
-	tree=buildTriangleTree(ltri,face_calc_bound(*ltri),4);
+	unsigned int maxdepth = (unsigned int)(8.0 + 1.8755035531556525*log((PFLOAT)triangles.size()));
+	tree=buildTriangleTree(ltri, maxdepth, face_calc_bound(*ltri),4);
 	recalcBound();
 
 	// backOrco, replace translation with (transformed!) bound center
@@ -403,14 +484,12 @@ point3d_t meshObject_t::toObject(const point3d_t &p)const
 	return (back*p);
 }
 
-
 vector3d_t meshObject_t::toObjectRot(const vector3d_t &v) const
 {
 	if (unt) return v;
 	return (backRot*v);
 }
 
-
 point3d_t meshObject_t::toObjectOrco(const point3d_t &p) const
 {
 	if (unt) return p;
@@ -601,7 +680,7 @@ static bool checkTriangleInPlane(triangle_t *t,point3d_t min,point3d_t max,int a
 /* suggestions and discussions on how to optimize code. */
 /* Thanks to David Hunt for finding a ">="-bug!         */
 /********************************************************/
-#include <math.h>
+//#include <math.h>
 #include <stdio.h>
 
 
@@ -765,13 +844,13 @@ bool triBoxOverlap(const bound_t &bound,const point3d_t &tria,
 #undef AXISTEST_Z12
 #undef AXISTEST_Z0
 
-tnode_t * buildTriangleTree(std::vector<triangle_t*> *v,
+tnode_t * buildTriangleTree(std::vector<triangle_t*> *v, unsigned int maxdepth,
 		const bound_t &bound,unsigned int dtol,unsigned int depth,unsigned int lostd,
 		PFLOAT &avgdepth,PFLOAT &avgsize)
 {
 	typedef std::vector<triangle_t*>::const_iterator vector_const_iterator;
 	
-	if((v->size()<=dtol) || (lostd>3))
+	if((v->size()<=dtol) || (lostd>3) || (depth>=maxdepth))
 	{
 		if(v->size())
 		{
@@ -826,7 +905,8 @@ tnode_t * buildTriangleTree(std::vector<triangle_t*> *v,
 	for(vector_const_iterator i=v->begin();i!=v->end();++i)
 	{
 		int pos=cheapPosition(**i,bound,media,axis);
-		//if(pos==trianglePosition_t::NONE) pos=expensivePosition(**i,bound,media,axis);
+		if(pos==trianglePosition_t::NONE) pos=expensivePosition(**i,bound,media,axis);
+		/*
 		if(pos==trianglePosition_t::NONE)
 		{
 			if(triBoxOverlap(bl,*((*i)->a),*((*i)->b),*((*i)->c))) pos=trianglePosition_t::LOWER;
@@ -836,21 +916,6 @@ tnode_t * buildTriangleTree(std::vector<triangle_t*> *v,
 				else pos=trianglePosition_t::HIGHER;
 			}
 		}
-		/*
-		if(pos!=alter) 
-		{
-			cout<<"wwawwwaaarning "<<axis<<" "<<media<<" "<<pos<<" "<<alter<<endl;
-			cout<<"triangle "<<*((*i)->a)<<" "<<*((*i)->b)<<" "<<*((*i)->c)<<endl;
-			point3d_t a,b;
-			bound.get(a,b);
-			cout<<"bound "<<a<<" "<<b<<endl;
-		}
-		*/
-		/*
-			if(triBoxOverlap(bl,*((*i)->a),*((*i)->b),*((*i)->c)))
-				vl->push_back(*i);
-			if(triBoxOverlap(br,*((*i)->a),*((*i)->b),*((*i)->c)))
-				vr->push_back(*i);
 		*/
 		if(pos==trianglePosition_t::LOWER) vl->push_back(*i);
 		else if(pos==trianglePosition_t::HIGHER) vr->push_back(*i);
@@ -861,16 +926,40 @@ tnode_t * buildTriangleTree(std::vector<triangle_t*> *v,
 		}
 	}
 
+	if(vl->empty() || vr->empty())
+	{
+		if(vl->empty())	media=minimize(*vr,bound,axis);
+		else media=maximize(*vl,bound,axis);
+		switch(axis)
+		{
+			case AXISX:
+				bl.setMaxX(media);
+				br.setMinX(media);
+				plane.set(bound,media,plane_t::XAXIS);
+				break;
+			case AXISY:
+				bl.setMaxY(media);
+				br.setMinY(media);
+				plane.set(bound,media,plane_t::YAXIS);
+				break;
+			case AXISZ:
+				bl.setMaxZ(media);
+				br.setMinZ(media);
+				plane.set(bound,media,plane_t::ZAXIS);
+				break;
+		}
+	}
+
 	int lostl=((vl->size()==v->size())) ? (lostd+1) : (lostd-1);
 	int lostr=((vr->size()==v->size())) ? (lostd+1) : (lostd-1);
 	if(lostl<0) lostl=0;
 	if(lostr<0) lostr=0;
 	delete v;
 	PFLOAT avgd,avgs;
-	tnode_t *left=buildTriangleTree(vl,bl,dtol,depth+1,lostl,avgd,avgs);
+	tnode_t *left=buildTriangleTree(vl, maxdepth, bl,dtol,depth+1,lostl,avgd,avgs);
 	avgdepth=avgd;
 	avgsize=avgs;
-	tnode_t *right=buildTriangleTree(vr,br,dtol,depth+1,lostr,avgd,avgs);
+	tnode_t *right=buildTriangleTree(vr, maxdepth, br,dtol,depth+1,lostr,avgd,avgs);
 	avgdepth+=avgd;
 	avgsize+=avgs;
 	avgdepth*=0.5;
@@ -993,11 +1082,11 @@ meshObject_t *meshObject_t::factory(const std::vector<point3d_t> &ver, const std
 }
 
 		
-meshObject_t *meshObject_t::factory(const matrix4x4_t &M, const std::vector<point3d_t> &ver,
+meshObject_t *meshObject_t::factory(bool _hasorco, const matrix4x4_t &M, const std::vector<point3d_t> &ver,
 		const std::vector<vector3d_t> &nor, const std::vector<triangle_t> &ts,
 		const std::vector<GFLOAT> &fuv, const std::vector<CFLOAT> &fvcol)
 {
-	return new meshObject_t(M,ver,nor,ts,fuv,fvcol);
+	return new meshObject_t(_hasorco, M, ver,nor,ts,fuv,fvcol);
 }
 
 __END_YAFRAY
