@@ -51,8 +51,9 @@ template<class T> class pureBspTree_t;
 class YAFRAYCORE_EXPORT meshObject_t : public object3d_t
 {
 	public:
-		void hasOrco(bool b) {hasorco=b;};
+		void hasOrco(bool b) { hasorco=b; }
 		void autoSmooth(PFLOAT angle);
+		void tangentsFromUV();
 		virtual ~meshObject_t();
 		virtual int type() const {return MESH;};
 		virtual void transform(const matrix4x4_t &m);
@@ -65,18 +66,19 @@ class YAFRAYCORE_EXPORT meshObject_t : public object3d_t
 
 		static meshObject_t *factory(const std::vector<point3d_t> &ver, const std::vector<vector3d_t> &nor,
 				        const std::vector<triangle_t> &ts, const std::vector<GFLOAT> &fuv, const std::vector<CFLOAT> &fvcol);
-		static meshObject_t *factory(const matrix4x4_t &M, const std::vector<point3d_t> &ver,
+		static meshObject_t *factory(bool _hasorco, const matrix4x4_t &M, const std::vector<point3d_t> &ver,
 				const std::vector<vector3d_t> &nor, const std::vector<triangle_t> &ts,
 				const std::vector<GFLOAT> &fuv, const std::vector<CFLOAT> &fvcol);
 
 	protected:
 		meshObject_t(const std::vector<point3d_t> &ver, const std::vector<vector3d_t> &nor,
 				const std::vector<triangle_t> &ts, const std::vector<GFLOAT> &fuv, const std::vector<CFLOAT> &fvcol);
-		meshObject_t(const matrix4x4_t &M, const std::vector<point3d_t> &ver,
+		meshObject_t(bool _hasorco, const matrix4x4_t &M, const std::vector<point3d_t> &ver,
 				const std::vector<vector3d_t> &nor, const std::vector<triangle_t> &ts,
 				const std::vector<GFLOAT> &fuv, const std::vector<CFLOAT> &fvcol);
 		meshObject_t()
-		{ unt=true;
+		{
+			unt=true;
 			shader=NULL;
 			tree=NULL;
 			hasorco=false;
@@ -85,7 +87,7 @@ class YAFRAYCORE_EXPORT meshObject_t : public object3d_t
 		
 		void recalcBound();
 		std::vector<point3d_t> vertices;
-		std::vector<vector3d_t> normals;
+		std::vector<vector3d_t> normals, tangents;
 		std::vector<triangle_t> triangles;
 		std::vector<GFLOAT> facesuv;
 		std::vector<CFLOAT> faces_vcol;
