@@ -23,7 +23,7 @@
 #define __OBJECT3D_H
 
 #ifdef HAVE_CONFIG_H
-#include<config.h>
+#include <config.h>
 #endif
 
 #include "matrix4.h"
@@ -33,6 +33,7 @@ __END_YAFRAY
 #include "surface.h"
 #include "shader.h"
 #include "bound.h"
+//#include "spectrum.h"
 
 __BEGIN_YAFRAY
 #define MESH 0
@@ -44,7 +45,13 @@ class YAFRAYCORE_EXPORT object3d_t
 	friend class photonLight_t;
 	public:
 		object3d_t()
-		{radiosity=true;rad_pasive=true;shadow=true;caus=false;tag=NULL;};
+		{
+			radiosity = true;
+			rad_pasive = true;
+			shadow = true;
+			caus = false;
+			tag = NULL;
+		}
 		virtual ~object3d_t() {};
 		virtual int type() const =0;
 		virtual void transform(const matrix4x4_t &m) =0;
@@ -64,12 +71,20 @@ class YAFRAYCORE_EXPORT object3d_t
 		void reciveRadiosity(bool r) {rad_pasive=r;};
 		bool castShadows() const  {return shadow;};
 		void castShadows(bool r) {shadow=r;};
-		bool caustics() const  {return caus;};
-		void caustics(bool r) {caus=r;};
-		void setCaustic(const color_t &r,const color_t &t,PFLOAT ior)
-								{caus_rcolor=r;caus_tcolor=t;caus_IOR=ior;};
-		void getCaustic(color_t &r,color_t &t,PFLOAT &ior)
-								{r=caus_rcolor;t=caus_tcolor;ior=caus_IOR;};
+		bool caustics() const  { return caus; }
+		void caustics(bool r) { caus=r; }
+		void setCaustic(const color_t &r, const color_t &t, PFLOAT ior)
+		{
+			caus_rcolor = r;
+			caus_tcolor = t;
+			caus_IOR = ior;
+		}
+		void getCaustic(color_t &r, color_t &t, PFLOAT &ior)
+		{
+			r = caus_rcolor;
+			t = caus_tcolor;
+			ior = caus_IOR;
+		}
 	protected:
 		shader_t *shader;
 		bool radiosity;
@@ -80,7 +95,6 @@ class YAFRAYCORE_EXPORT object3d_t
 		color_t caus_rcolor;
 		color_t caus_tcolor;
 		PFLOAT caus_IOR;
-
 };
 
 template<class T> class geomeTree_t;
