@@ -26,12 +26,16 @@
 #include<config.h>
 #endif
 
-#if HAVE_PTHREAD
-#include<pthread.h>
-#include <semaphore.h>
-#include<map>
-#include <unistd.h>
-#endif
+#if HAVE_PTHREAD 
+#include <pthread.h> 
+#include <semaphore.h> 
+#include <map> 
+#ifdef WIN32 
+#include <winunistd.h> 
+#else 
+#include <unistd.h> 
+#endif 
+#endif 
 
 #include <vector>
 #include <list>
@@ -61,6 +65,8 @@ struct YAFRAYCORE_EXPORT renderState_t
 	PFLOAT traveled;
 	int pixelNumber;
 	point3d_t screenpos;
+	bool chromatic;
+	PFLOAT cur_ior;
 
 	protected:
 		renderState_t(const renderState_t &r) {};//forbiden
@@ -99,8 +105,6 @@ class YAFRAYCORE_EXPORT scene_t
 		void addFilter(filter_t *filter);
 		color_t raytrace(renderState_t &state,const point3d_t &from,
 				const vector3d_t & ray)const;
-		color_t raytrace_indir(renderState_t &state,const point3d_t &from, 
-				const vector3d_t & ray, bool isdif=false) const;
 		bool isShadowed(renderState_t &state,const surfacePoint_t &p,
 				const point3d_t &l)const;
 		bool isShadowed(renderState_t &state,const surfacePoint_t &p,
@@ -124,10 +128,7 @@ class YAFRAYCORE_EXPORT scene_t
 				std::vector<CFLOAT> &dep,
 				std::vector<CFLOAT> &alpha,int numline,int pass)
 		{
-			if (AA_onepass_max)
-                                return doAllPasses(state, line, dep, alpha, numline);
-			else
-                                return doOnePass(state, line, dep, alpha, numline, pass);
+			return doOnePass(state, line, dep, alpha, numline, pass);
 		};
 
 		virtual void renderPart(colorOutput_t &out, int curpass, int off);
@@ -149,43 +150,43 @@ class YAFRAYCORE_EXPORT scene_t
 		PFLOAT selfBias()const {return self_bias;};
 
 		void setBackground(const background_t *b) {background=b;};
-		const color_t  getBackground(const vector3d_t &dir,renderState_t &state)const
-			{if(background==NULL) return color_t(0.0); else return (*background)(dir,state);};
-
-                void setCPUs(const int num) {cpus = num;};
-
-		// indirect sampling
-		void setIndirect(int indir_sam, PFLOAT indir_pow)
+		const color_t getBackground(const vector3d_t &dir, renderState_t &state, bool filtered=false) const
 		{
-			indirect_grid = int(sqrt((PFLOAT)indir_sam));
-			indirect_gdiv = 1.0/(PFLOAT)indirect_grid;
-			indirect_gdiv2pi = indirect_gdiv*M_PI*2.0;
-			indirect_samples = indir_sam;
-			indirect_power = indir_pow/PFLOAT(indir_sam);
+			if (background==NULL) return color_t(0.0);
+			return (*background)(dir, state, filtered);
 		}
 
+		void setCPUs(const int num) { cpus = num; }
+
 		// gamma & exposure
 		void setGamma(CFLOAT g) { gamma_R=0.0;  if (g!=0.0) gamma_R=1.0/g; }
 		void setExposure(CFLOAT e) { exposure = -e; }
-		void adjustColor(color_t &c)const {c.expgam_Adjust(exposure, gamma_R);};
+		void adjustColor(color_t &c)const { c.expgam_Adjust(exposure, gamma_R, clamp_rgb); }
 
 		// fog
 		void setFog(CFLOAT dens, const color_t &col) { fog_density=dens;  fog_color=col; }
 		void fog_addToCol(CFLOAT depth, color_t &curcol) const;
 
 		// new AA
-		void setAASamples(int ps, int ms, PFLOAT pw, PFLOAT th, int spmax)
+		void setAASamples(int ps, int ms, PFLOAT pw, PFLOAT th, bool jf=false)
 		{
 			AA_passes = ps;
 			AA_minsamples = ms;
 			AA_samdiv = 1.0/(PFLOAT)ms;
 			AA_pixelwidth = pw;
 			AA_threshold = th;
-			AA_onepass_max  = spmax;
-			HSEQ1.setBase(2);
-			HSEQ2.setBase(3);
+			AA_jitterfirst = jf;
 		}
 
+		// for LDR output, it is useful to clamp light values in AA sampling
+		// so that AA will look better in parts of the image where fast high contrast differences occur
+		void clampRGB(bool cr) { clamp_rgb=cr; }
+		// for float output, bypass tone mapping
+		void tonemap(bool tm) { do_tonemap=tm; }
+		// switch to enable/disable alpha premultiply, and background masking
+		void alphaPremultiply(bool ap) { alpha_premultiply=ap; }
+		void alphaMaskBackground(bool abm) { alpha_maskbackground=abm; }
+
 		void setRepeatFirst() {repeatFirst=true;};
 		bool getRepeatFirst()const {return repeatFirst;};
 		PFLOAT getWorldResolution()const {return world_resolution;};
@@ -221,7 +222,7 @@ class YAFRAYCORE_EXPORT scene_t
 		scene_t(const scene_t &s) {}; //forbiden
 
 		camera_t *render_camera;
-                int cpus;
+		int cpus;
 		PFLOAT min_raydis;
 		PFLOAT world_resolution;
 		std::list<object3d_t *> obj_list;
@@ -238,25 +239,22 @@ class YAFRAYCORE_EXPORT scene_t
 		geomeTree_t<object3d_t> *BTree;
 		PFLOAT self_bias;
 		const background_t *background;
-		// indirect lighting parameters
-		int indirect_samples, indirect_grid;
-		CFLOAT indirect_power;
-		PFLOAT indirect_gdiv, indirect_gdiv2pi;
 		// exposure and gamma controls
 		CFLOAT gamma_R, exposure;
 		// simple fog, density & color
 		CFLOAT fog_density;
 		color_t fog_color;
 		// new AA parameters
-		int AA_onepass_max;
 		int AA_passes, AA_minsamples;
+		bool AA_jitterfirst;
 		PFLOAT AA_pixelwidth, AA_threshold, AA_samdiv;
-		Halton HSEQ1, HSEQ2;
 		// used to keep track of the screen sampling position, for 'win' texmap mode
 		//point3d_t screenpos;
 		PFLOAT scymin,scymax,scxmin,scxmax;
 		bool repeatFirst;
 		std::map<std::string,const void *> published;
+		bool do_tonemap, clamp_rgb;
+		bool alpha_premultiply, alpha_maskbackground;
 };
 
 __END_YAFRAY
