@@ -28,11 +28,16 @@
 
 #include<cmath>
 #include<iostream>
-#include<stdlib.h>
-#include<stdio.h>
+//#include<stdlib.h>
+//#include<stdio.h>
 
 __BEGIN_YAFRAY
 
+// useful trick found in trimesh2 lib by Szymon Rusinkiewicz
+// makes code for vector dot- cross-products much more readable
+#define VDOT *
+#define VCROSS ^
+
 class YAFRAYCORE_EXPORT vector3d_t
 {
 	public:
@@ -47,7 +52,7 @@ class YAFRAYCORE_EXPORT vector3d_t
 			PFLOAT vl = x*x + y*y + z*z;
 			if (vl!=0.0) {
 				vl = sqrt(vl);
-				PFLOAT d = 1.0/vl;
+				const PFLOAT d = 1.0/vl;
 				x*=d; y*=d; z*=d;
 			}
 			return vl;
@@ -57,7 +62,7 @@ class YAFRAYCORE_EXPORT vector3d_t
 		{
 			PFLOAT vl = x*x + y*y + z*z;
 			if (vl!=0.0) {
-				PFLOAT d = 1.0/sqrt(vl);
+				const PFLOAT d = 1.0/sqrt(vl);
 				x*=d; y*=d; z*=d;
 			}
 			return vl;
@@ -212,7 +217,7 @@ inline void vector3d_t::normalize()
 
 inline vector3d_t reflect(const vector3d_t &n,const vector3d_t &v)
 {
-	PFLOAT vn = v*n;
+	const PFLOAT vn = v*n;
 	if (vn<0) return -v;
 	return 2*vn*n - v;
 }
@@ -241,14 +246,15 @@ inline void createCS(const vector3d_t &N, vector3d_t &u, vector3d_t &v)
 	}
 	else
 	{
-		PFLOAT d = N.y*N.y + N.x*N.x;
-		if (d!=0.0) d=1.0/sqrt(d);
+		// Note: The root cannot become zero if
+		// N.x==0 && N.y==0.
+		const PFLOAT d = 1.0/sqrt(N.y*N.y + N.x*N.x);
 		u.set(N.y*d, -N.x*d, 0);
 		v = N^u;
 	}
 }
 
-YAFRAYCORE_EXPORT void ShirleyDisc(PFLOAT r1, PFLOAT r2, PFLOAT &u, PFLOAT &v);
+YAFRAYCORE_EXPORT void ShirleyDisk(PFLOAT r1, PFLOAT r2, PFLOAT &u, PFLOAT &v);
 
 extern YAFRAYCORE_EXPORT int myseed;
 
@@ -301,7 +307,6 @@ inline vector3d_t RandomSpherical()
 	return v;
 }
 
-
 YAFRAYCORE_EXPORT vector3d_t randomVectorCone(const vector3d_t &D, const vector3d_t &U, const vector3d_t &V,
 						PFLOAT cosang, PFLOAT z1, PFLOAT z2);
 YAFRAYCORE_EXPORT vector3d_t randomVectorCone(const vector3d_t &dir, PFLOAT cosangle, PFLOAT r1, PFLOAT r2);
